Class: AutoC::HashSet

Inherits:
Set show all
Defined in:
lib/autoc/hash_set.rb

Direct Known Subclasses

AutoC::HashMap::HashSet

Defined Under Namespace

Classes: Range

Constant Summary

Constants inherited from Composite

Composite::CAMEL_CASE_DECORATOR, Composite::DEFINITIONS, Composite::PRIVATE, Composite::SNAKE_CASE_DECORATOR

Constants included from Entity

Entity::ReferenceSet

Constants included from STD

STD::ASSERT_H, STD::BOOL, STD::CHAR, STD::COMPLEX, STD::COMPLEX_H, STD::DOUBLE, STD::DOUBLE_COMPLEX, STD::DOUBLE_T, STD::FLOAT, STD::FLOAT_COMPLEX, STD::FLOAT_T, STD::INT, STD::INTMAX_T, STD::INTPTR_T, STD::INTTYPES_H, STD::LONG, STD::LONG_DOUBLE, STD::LONG_DOUBLE_COMPLEX, STD::LONG_LONG, STD::MALLOC_H, STD::MATH_H, STD::PTRDIFF_T, STD::SHORT, STD::SIGNED_CHAR, STD::SIZE_T, STD::STDBOOL_H, STD::STDDEF_H, STD::STDLIB_H, STD::STRING_H, STD::UINTMAX_T, STD::UINTPTR_T, STD::UNSIGNED, STD::UNSIGNED_CHAR, STD::UNSIGNED_LONG, STD::UNSIGNED_LONG_LONG, STD::UNSIGNED_SHORT, STD::WCHAR_T

Instance Attribute Summary

Attributes inherited from Collection

#element

Attributes inherited from Composite

#_master, #visibility

Attributes inherited from Type

#signature

Instance Method Summary collapse

Methods inherited from Set

#orderable?

Methods inherited from Collection

#comparable?, #copyable?, #destructible?, #hashable?, new, #orderable?, #type_tag

Methods inherited from Composite

allocator, allocator=, #const_lvalue, #const_rvalue, decorator, decorator=, #defgroup, hasher, #hasher, hasher=, #identifier, #ingroup, #inspect, #lvalue, #memory, new, #prefix, #public?, #respond_to_missing?, #rvalue, #to_value, #type_tag

Methods included from Entity

#<=>, #complexity, #dependencies, #forward_declarations, #implementation, #interface, #position, #references, #total_dependencies, #total_references

Methods inherited from Type

abstract, #comparable?, #constructible?, #copy, #copyable?, #custom_constructible?, #custom_create, #default_constructible?, #default_create, #destroy, #destructible?, #hashable?, #inspect, #orderable?, #to_s, #to_type

Constructor Details

#initialize(*args, **kws) ⇒ HashSet



27
28
29
30
# File 'lib/autoc/hash_set.rb', line 27

def initialize(*args, **kws)
  super
  dependencies << _bin
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class AutoC::Composite

Instance Method Details

#_binObject



25
# File 'lib/autoc/hash_set.rb', line 25

def _bin = @_bin ||= _bin_class.new(identifier(:_vector, abbreviate: true), _slot, _master: self, visibility: :internal)

#_bin_classObject



19
# File 'lib/autoc/hash_set.rb', line 19

def _bin_class = Vector

#_find_slot_hash(hash) ⇒ Object



97
98
99
100
101
102
# File 'lib/autoc/hash_set.rb', line 97

def _find_slot_hash(hash)
  %{
    assert(target);
    return #{_bin.view.('target->bin', "#{hash} & target->hash_mask")};
  }
end

#_slotObject



23
# File 'lib/autoc/hash_set.rb', line 23

def _slot = @_slot ||= _slot_class.new(identifier(:_list, abbreviate: true), element, _master: self, maintain_size: false, visibility: :internal)

#_slot_classObject



17
# File 'lib/autoc/hash_set.rb', line 17

def _slot_class = List

#rangeObject



21
# File 'lib/autoc/hash_set.rb', line 21

def range = @range ||= Range.new(self, visibility: visibility)

#render_interface(stream) ⇒ Object



32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/autoc/hash_set.rb', line 32

def render_interface(stream)
  if public?
    stream << %{
      /**
        #{defgroup}

        @brief Unordered collection of unique elements of type #{element}

        For iteration over the set elements refer to @ref #{range}.

        @see C++ [std::unordered_set<T>](https://en.cppreference.com/w/cpp/container/unordered_set)

        @since 2.0
      */
      /**
        #{ingroup}
        @brief Opaque structure holding state of the hash set
        @since 2.0
      */
    }
  else
    stream << PRIVATE
  end
  stream << %{
    typedef struct {
      #{_bin} bin; /**< @private */
      size_t size; /**< @private */
      size_t hash_mask; /**< @private */
    } #{signature};
  }
end